package database;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
//import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Random;

import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import database.Question.Type;

public class DataBase {
  private ArrayList<Question> questions = new ArrayList<Question>();
  private ArrayList<Integer> history = new ArrayList<Integer>();
  private int position=0;
  private int min=0;
  private int max=1;
  private boolean excludesingle=false;
  private boolean excludemulti=false;
  private boolean excluderegexp=false;
  private boolean sorted=true;
  
  private ArrayList<ActionListener> action = new ArrayList<ActionListener>();
  
  public void addQuestion(Question q)
  { questions.add(q); }
  
  //#########################################################

  public void excludeSingle(boolean b){ excludesingle=b; }
  public void excludeMulti(boolean b){ excludemulti=b; }
  public void excludeRegexp(boolean b){ excluderegexp=b; }
  public boolean isSingleExcuded(){ return excludesingle; }
  public boolean isMultiExcuded(){ return excludemulti; }
  public boolean isRegexpExcuded(){ return excluderegexp; }
  
  //#########################################################

  public void setRange(int mn, int mx)
  {
    if(mn>-1 && mn<size()) min=mn;
    if(mx>0 && mx<size()) max=mx;
    if(min>max){ int m =min; min=max; max=m; }
  }
  
  public int getMin(){ return min;}
  public int getMax(){ return max;}
  public void setMin(int m){ if(m>-1 && m<size() && m < max) min=m;}
  public void setMax(int m){ if(m>0 && m<size() && m > min) max=m;}

  //#########################################################
  
  public int getPosition(){ return position; }
  public void setPosition(int pos){ if(pos >=min && pos <=max){ history.add(position); position=pos; } }

  //#########################################################
  
  public void deleteHistory(){ history = new ArrayList<Integer>(); }
  public void resetPosition(){position=min;}
  public void deleteUserAnswers(){ for(Question q: questions) q.deleteUserAnswers(); }

  //#########################################################
  public int size(){ return questions.size();}
  //#########################################################

  public boolean QuestionExists(String id)
  { return getQuestionPosition(id)>-1? true: false;}
  
  public boolean QuestionExists(int pos)
  { return (pos >-1 && pos<questions.size())? true: false; }
  
  public int getQuestionPosition(String id)
  {
    for(int i=0; i<questions.size();i++) if(questions.get(i).getId().equals(id)) return i;
    return -1;
  }
  
  //##########################################################

  public Question getQuestion()
  { return getQuestion(position); }

  public Question getQuestion(int pos)
  { return QuestionExists(pos)? questions.get(pos): null; }
  
  public Question getQuestionById(String id)
  { return getQuestion(getQuestionPosition(id)); }
  
  public Question[] getAllQuestions()
  {
    ArrayList<Question> q=new ArrayList<Question>();
    for(int i=min; i<=max; i++) q.add(getQuestion(i));
    Collections.sort(q);
    return q.toArray(new Question[q.size()]);
  }

  public Question[] getAllPossibleQuestions()
  {
    ArrayList<Question> q=new ArrayList<Question>();
    for(int i=min; i<=max; i++) if(isPossibleQuestion(i)) q.add(getQuestion(i));
    Collections.sort(q);
    return q.toArray(new Question[q.size()]);
  }

  public void sortQuestions()
  {
    Collections.sort(questions);
    sorted=true;
  }
  
  public boolean isSortedQuestions(){return sorted; }
  public boolean isSortedAnswers()
  {
    boolean s=true;
    for(Question q:questions) if(!q.isSortedAnswers()) s=false;
    return s;
  }
  
  public void sortAnswers()
  { for(Question q:questions) q.sortAnswers(); }

  public void randomQuestions()
  {
    Random rn = new Random();
    for(int p1=min; p1<max; p1++)
    {
      int p2=(int)(rn.nextDouble()*(max-min))+min;
      Question a=questions.get(p1);
      questions.set(p1,questions.get(p2));
      questions.set(p2,a);
    }
    sorted=false;
  }
  
  public void randomAnswers()
  { for(Question q:questions) q.randomAnswers(); }
  
  //#########################################################
  
  private boolean isPossibleQuestion(int pos)
  { return isPossibleQuestion(pos,!excludesingle,!excludemulti,!excluderegexp); }
  
  public boolean isPossibleQuestion(int pos, boolean single, boolean multi, boolean regexp)
  {
    if( QuestionExists(pos) &&
        (
          (single && questions.get(pos).getType()==Type.single) ||
          (multi  && questions.get(pos).getType()==Type.multi) ||
          (regexp && questions.get(pos).getType()==Type.regexp)
        )
      )return true;
    return false;
  }
  
  public int countAllPossibleQuestions(int min, int max, boolean single, boolean multi, boolean regexp)
  {
    int ret=0;
    for(int i=min; i<=max; i++)
    { if(isPossibleQuestion(i,single,multi,regexp)) ret++; }
    return ret;
  }

  public int countAllPossibleQuestions(int min, int max)
  { return countAllPossibleQuestions(min, max, !excludesingle,!excludemulti,!excluderegexp); }

  
  private int getNextQuestionPositionTo(int pos)
  {
    for(; pos<questions.size(); pos++) if(isPossibleQuestion(pos)) break;
    return pos; 
  }
  
  private int getPreviousQuestionPositionTo(int pos)
  {
    for(int ret=pos-1; ret>min; ret--) if(isPossibleQuestion(ret)) return ret;
    if(isPossibleQuestion(min)) return min;
    return pos; 
  }
  
  //#########################################################
  public void start()
  {
    deleteHistory();
    deleteUserAnswers();
    resetPosition();
    setPosition(getNextQuestionPositionTo(position));
  }

  public void next()
  {
    //System.out.println("HISTORY A: "+history);
    int pos=getNextQuestionPositionTo(position+1);
    if(pos <= max) setPosition(pos);

    if(getNextQuestionPositionTo(pos+1) > max) doAction("DBQLAST",this);
    else doAction("DBQMIDDLE",this);

    //System.out.println("HISTORY NEXT: "+history);
  }
  
  public void previous()
  {
    //System.out.println("HISTORY C: "+history);
    int pos=getPreviousQuestionPositionTo(position);
    if(pos >= min) setPosition(pos);

    if(getPreviousQuestionPositionTo(pos) <= min) doAction("DBQFIRST",this);
    else doAction("DBQMIDDLE",this);

    //System.out.println("HISTORY PREV: "+history);
  }
  
  public void first()
  {
    setPosition(getNextQuestionPositionTo(min));
    doAction("DBQFIRST",this);
    //System.out.println("HISTORY FIRST: "+history);
  }
  
  public void last()
  {
    //System.out.println("HISTORY G: "+history);
    setPosition(getPreviousQuestionPositionTo(max+1));
    doAction("DBQLAST",this);
    //System.out.println("HISTORY LAST: "+history);
  }

  
  public static DataBase loadDB(File f) throws IOException
  { return parseDBString(slurpFile(f)); }
  
  public static DataBase loadXDB(File f) throws IOException
  {
    DataBase db=new DataBase();
    Document tree=null;
    try
    { tree = DocumentBuilderFactory.newInstance().newDocumentBuilder().parse(f); }
    catch (Exception ex)
    { tree=null; }
    
    if(tree != null)
    {
      NodeList fragen = tree.getElementsByTagName("question");
      for(int i=0; i<fragen.getLength();i++)
      {
        Element frage = (Element)(fragen.item(i));

        // Typ der Frage bestimmen
        String stype=frage.getElementsByTagName("answers").item(0).getAttributes().getNamedItem("type").getNodeValue();
        Type typ = Type.regexp;
        if(stype.contains("single"))
        {typ = Type.single; }
        else if(stype.contains("multi"))
        { typ = Type.multi; }
        //System.out.println("IN-TYPE: "+stype+ " --> "+typ);
        
        // "Name" der Frage
        String id = frage.getElementsByTagName("nr").item(0).getTextContent();

        // Anzahl der mglichen Punkte, bei richtiger Antwort
        int punkte = Integer.parseInt(fragen.item(i).getAttributes().getNamedItem("points").getNodeValue());
        
        // Fragenblock lesen
        String frag = frage.getElementsByTagName("text").item(0).getTextContent();

        // Mgliche Antworten lesen
        NodeList antworten = ((Element)(frage.getElementsByTagName("answers").item(0))).getElementsByTagName("answer");
        Answer[] antw = new Answer[antworten.getLength()];
        for(int ii=0; ii<antworten.getLength();ii++)
        {
          Node an=antworten.item(ii);
          //System.out.println(an.toString());
          String txt=an.getTextContent();
          boolean wahr=false;
          NamedNodeMap attr = an.getAttributes();
          Node stat=attr.getNamedItem("status");
          if(stat.getNodeValue().contains("correct")) wahr=true;
          //System.out.println("IS WAHR: "+stat.getNodeValue()+ " --> "+wahr);
          if(typ == Type.regexp) wahr=true;
          antw[ii]=new Answer(ii,txt,wahr,typ);
        }

        // Erklhrungsblock lesen
        
        String expl="";
        
        if(frage.getElementsByTagName("explanation") != null && frage.getElementsByTagName("explanation").item(0) != null)
        { expl = frage.getElementsByTagName("explanation").item(0).getTextContent(); }

        db.addQuestion(new Question(i,id,punkte,frag,antw,typ,expl));
      }
    }
    db.setRange(0,db.size()-1);
    return db;
  }

  
  public static DataBase parseDBString(String s)
  {
    if(s==null) return null;
    //System.out.println(s);
    
    String[] data=s.split("QUESTION\\s*");
    
    //System.out.println(data[1]);
    
    DataBase db =new DataBase();
    int pos=0;
    for(String q: data)
    {
      if(!q.matches("\\A[\r\n\\s]*\\z"))
      {
        String[] l=q.split("\\s*:\\s*?\n?",2);
        
        Type type=null;

        String id=l[0];
        //System.out.println("Parse Frage: "+id);
        q=l[1];

        l=q.split("(Answer|ANSWER|answer)\\s*:\\s*",2);

        String qtext="";
        boolean noregexp=false;
        for(String ll: l[0].split("\n"))
        {
          //System.out.println("-->"+ll);
          if(!ll.matches("\\A\\p{Upper}\\..+"))
          {
            //System.out.println("---->"+ll);
            qtext+=ll+"\n";
          }
          else
          { noregexp=true; }
        }
        //System.out.println(qtext);
        q=l[1];

        HashMap<Integer,Integer> answers_unique = new HashMap<Integer,Integer>();
        ArrayList<String> answers = new ArrayList<String>(); 
        if(noregexp)
        {
          int i=-1;
          for(String ll: l[0].split("\n"))
          {
            //System.out.println("-->"+ll);
            if(ll.matches("\\A\\p{Upper}\\..+"))
            {
              int key=ll.charAt(0);
              if(answers_unique.get(key) == null)
              {
                i++;
                //System.out.println("ID: "+id+" => CHANGE: "+i+" -> "+(char)key);
                answers_unique.put(key,i);
              }
              while(i>=answers.size()) answers.add("");
              if( answers.get(i) == null ) answers.add(i,"");
              
              String[] a=ll.split("\\.\\s",2);
              if(a.length>1) answers.set(i,answers.get(i)+a[1]+"\n");
            }
          }
        }
        
        else type=Type.regexp;
        
        l=q.split("[\r\n\\s]*Explanation\\s*:?\\s*");
        boolean[] correct=new boolean[answers.size()];
        if(type == null)
        {
          String[] c=l[0].split("\\s*,\\s*");
          if(c.length>1) type=Type.multi;
          //if(type == Type.multi) System.out.println(id+" ==> "+l[0]);
          else if(c.length == 1) type=Type.single;
          for(int i: answers_unique.keySet())
          {
            correct[answers_unique.get(i)]=false;
            for(String cc: c)
            {
              if(i == cc.charAt(0))
              {
                correct[answers_unique.get(i)]=true;
                break;
              }
            }
          }
        }
        else
        {
          answers = new ArrayList<String>();
          answers.add(l[0]);
          correct = new boolean[]{false};
        }
        String desc=(l.length>1)?l[1]:"";
        int score=3;
        
        db.addQuestion(new Question(pos,id,score,qtext,answers.toArray(new String[answers.size()]),correct,type,desc));
        pos++;
      }
    }
    db.setRange(0,db.size()-1);
    return db;
  }

  private static String slurpFile(File file) throws IOException
  {
    String data="";
    InputStreamReader in=null;
    //System.out.println(file);
    if(file != null)
    {
      URL rsurl=DataBase.class.getResource("/"+file);
      if(rsurl!=null)
      {
        in = new InputStreamReader(rsurl.openStream(),"UTF8");
        //System.out.println("A "+rsurl+"  ->  "+in);
      }
      else 
      {
        in = new InputStreamReader(new FileInputStream(file),"UTF8");
        //System.out.println("B "+file+"  ->  "+in);
      }
    }  

    if(in!=null)
    {
      char[] dt = new char[(int)file.length()];
      //System.out.println("lese Datei ... ");
      int ok=in.read(dt,0,(int)file.length());
      //System.out.println("OK="+ok);
      data=String.copyValueOf(dt,0,ok);
      in.close();
      //System.out.println("\nabgeschlossen.");
    }
    else
      throw new IOException("Datei konnte nicht geffnet werden");
    return data;
  }

  public void addActionListener(ActionListener l){action.add(l);}
  public void removeActionListener(ActionListener l){action.remove(l);}
  private void doAction(String name,Object o)
  { for(ActionListener l:action) l.actionPerformed(new ActionEvent(o, 0, name)); }
}
