package database;

import java.util.Arrays;
import java.util.Random;

public class Question implements Comparable<Question>{

  public enum Type {single, multi, regexp};
  private String id;
  private String question;
  private String description;
  private Answer[] answers;
  private int ponints;
  private Type type;
  private int position=-1;
  private boolean sorted=true;
  
  public class allData{
    public int pos=0;
    public String id="";
    public String question = "";
    public String description = "";
    public String[] answers = new String[0];
    public boolean[] correct = new boolean[0];
    public int points=0;
    public String type="";
    allData(){};
    allData(int po, String i, String f, String[] a, boolean[] c, String d, int p, String t)
    { pos=po; id=i; question=f; answers=a; correct=c; description=d; points=p; type=t; }
  }
  
  public Question (int pos, String inid, Integer p, String f, Answer[] a, Type t, String b)
  {
    position=pos;
    id=inid;
    ponints=p;
    question= f;
    description = b;
    answers = a;
    type = t;
  }
  
  public Question (int pos,String inid, int p, String f, String[] a, boolean[] b, Type t, String d)
  { this(pos, inid, p, f, StringArraytoAnswer(a,b,t), t, d); }
  
  public Question (int pos, String id, int p, String f, String[] a, boolean[] b, String t, String d)
  { this(pos, id,p,f,a,b,StringToType(t),d);}

  public Question(allData d)
  { this(d.pos, d.id, d.points, d.question, StringArraytoAnswer(d.answers,d.correct,StringToType(d.type)), StringToType(d.type), d.description); }
  
  private static Type StringToType(String s)
  {
    if(s.equals("single")) return Type.single;
    else if(s.equals("multi")) return Type.multi;
    else return Type.regexp;
  }
  
  private static Answer[] StringArraytoAnswer(String[] a, boolean[] b, Type type)
  {
    Answer[] answers = new Answer[a.length];
    if( type != Type.regexp)
    {
      for(int i=0; i<answers.length; i++)
      {
        boolean bb=false;
        if(b!= null && b.length>i) bb=b[i];
        answers[i]= new Answer(i,a[i],bb,type);
      }
    }
    else
    {
      for(int i=0; i<answers.length; i++)
        answers[i]=new Answer(i,a[i],true,type);
    }
    return answers;
  }

  public String getQuestionText()
  { return this.question; }

  public String getDescriptionText()
  { return this.description; }

  public Type getType()
  { return this.type; }
  
  public boolean isAnswered()
  {
    for(Answer a: answers)  if(a.isAnswered()) return true;
    return false;
  }
  
  public void sortAnswers()
  {
    Arrays.sort(answers);
    sorted=true;
  }
  
  public void randomAnswers()
  {
    Random rn = new Random();
    for(int p1=0; p1<answers.length; p1++)
    {
      int p2=(int)(rn.nextDouble()*answers.length);
      Answer a=answers[p1];
      answers[p1]=answers[p2];
      answers[p2]=a;
    }
    sorted=false;
  }
  
  public boolean isSortedAnswers(){return sorted;}
 
  public String getTypeAsString()
  { return this.type.toString(); }
  
  public Answer[] getAnswers()
  { return this.answers; }
  
  public String[] getAnswersAsString()
  {
    String[] out = new String[this.answers.length];
    for( int i=0; i<this.answers.length; i++)
    { out[i]=this.answers[i].getText(); }
    return out;
  }
  
  public boolean[] getTrueAnswers()
  {
    boolean[] out = new boolean[this.answers.length];
    for( int i=0; i<this.answers.length; i++)
    { out[i]=this.answers[i].isOK(); }
    return out;
  }
  
  public String getTrueAnswersAsString()
  {
    String out = "";
    for( int i=0; i<this.answers.length; i++)
    { if(this.answers[i].isOK()) out+=this.answers[i].getText()+"\n"; }
    return out;
  }
  
  public int getPoints()
  { return this.ponints; }

  public String getId()
  { return this.id; }
  
  public allData getAll()
  { return new allData(position,id,question,getAnswersAsString(),getTrueAnswers(),description,ponints,getTypeAsString()); }

  //#############################################################
  public void setUserAnswers(boolean[] a)
  {
    System.out.println("USER "+a);
    if(getType()!=Type.regexp)
    {
      for(int i=0; i<a.length; i++)
      { answers[i].setUserSelection(a[i]); }
    }
  }
  
  public void setUserAnswerString(String a)
  {
    System.out.println("USER "+a);
    if(getType()==Type.regexp && answers != null)
    { answers[0].setUserText(a); }
  }
  
  public boolean[] getUserAnswers()
  {
    if(answers != null && isAnswered())
    {
      boolean[] ba=new boolean[answers.length];
      for(int i=0; i<answers.length; i++)
      { ba[i]=answers[i].getUserSelection();}
      return ba;
    }
    return null;
  }
  
  public String getUserAnswerString()
  { return (answers != null && isAnswered())? answers[0].getUserText(): ""; }
  
  public void deleteUserAnswers()
  {
    if(answers!=null)
    { for(Answer a: answers) a.resetUser(); }
  }
  
  public boolean testUserAnswers()
  {
    if(!isAnswered() ) return false;
    boolean is_ok = true;
    for(int i = 0; i < answers.length; i++)
    { if(!answers[i].isUserOK())is_ok=false; }
    return is_ok;
  }

  public int compareTo(Question q)
  { return position-q.position; }
}
