package gui;

import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import database.DataBase;

public class DBConfPanel extends JPanel implements ActionListener{
  private static final long serialVersionUID = -8978737207601237018L;
  private DataBase db=null;
  
  private JCheckBox selsingle;
  private JCheckBox selmulti;
  private JCheckBox selregexp;
  private JCheckBox selrandq;
  private JCheckBox selranda;
  
  private JSpinner spinnmin;
  private JSpinner spinnmax;
  private JLabel anz =new JLabel("");
  private JLabel summsingle =new JLabel("");
  private JLabel summmulti =new JLabel("");
  private JLabel summregexp =new JLabel("");
  private JLabel summ =new JLabel("");
  
  public DBConfPanel(DataBase d)
  {
    db=d;
    if(db!= null) init();
  }
  
  private void  init()
  {
    setLayout(new GridBagLayout());
    GridBagConstraints c = new GridBagConstraints();
    c.gridwidth=1;
    c.gridheight=1;
    c.fill = GridBagConstraints.HORIZONTAL;
    c.anchor = GridBagConstraints.FIRST_LINE_START;
    c.insets = new Insets(10,0,10,0);
    c.weightx=1.0;
    c.weighty=0.0;
    c.gridx=0;
    c.gridy=0;
    
    add(new JLabel("Gesammtzahl der Fragen im Datensatz: "+db.size()),c);
    
    c.gridy=1;
    add(new JLabel("Fragenbereich Auswhlen:"),c);

    c.gridy=2;
    {
      JPanel jp= new JPanel();
      add(jp,c);
      jp.setLayout(new GridBagLayout());
      GridBagConstraints jpc = new GridBagConstraints();
      jpc.gridwidth=1;
      jpc.gridheight=1;
      jpc.fill = GridBagConstraints.HORIZONTAL;
      jpc.anchor = GridBagConstraints.FIRST_LINE_START;
      jpc.insets = new Insets(10,10,10,10);
      jpc.weightx=0.0;
      jpc.weighty=0.0;
      jpc.gridx=0;
      jpc.gridy=0;

      jp.add(new JLabel(" von:"),jpc);
      
      jpc.weightx=1.0;
      jpc.gridx=1;
      spinnmin=createSpinner("CWSPINNMIN",db.getMin()+1,1,db.size());
      jp.add(spinnmin,jpc);

      jpc.weightx=0.0;
      jpc.gridx=2;
      jp.add(new JLabel(" bis:"),jpc);

      jpc.weightx=1.0;
      jpc.gridx=3;
      spinnmax=createSpinner("CWSPINNMAX",db.getMax()+1,2,db.size());
      jp.add(spinnmax,jpc);

      jpc.weightx=0.0;
      jpc.gridx=4;
      jp.add(new JLabel("   Anzahl:"),jpc);
      
      jpc.weightx=1.0;
      jpc.gridx=5;
      jp.add(anz,jpc);
    }
    
    c.gridy=3;
    {
      JPanel jp= new JPanel();
      add(jp,c);
      jp.setLayout(new GridBagLayout());
      GridBagConstraints jpc = new GridBagConstraints();
      jpc.gridwidth=1;
      jpc.gridheight=1;
      jpc.fill = GridBagConstraints.HORIZONTAL;
      jpc.anchor = GridBagConstraints.LINE_START;
      jpc.insets = new Insets(0,0,0,0);
      jpc.weighty=0.0;

      
      jpc.gridy=0;
      jpc.gridx=0;
      jpc.weightx=0.0;
      selsingle=createCheckBox("Single Choice Fragen","CWSELSINGLE",!db.isSingleExcuded());
      jp.add(selsingle,jpc);
      
      jpc.weightx=1.0;
      jpc.gridx=1;
      jp.add(summsingle,jpc);


      jpc.gridy=1;
      jpc.gridx=0;
      jpc.weightx=0.0;
      selmulti=createCheckBox("Multiple Choice Fragen","CWSELMULTI",!db.isMultiExcuded());
      jp.add(selmulti,jpc);
      
      jpc.gridx=1;
      jpc.weightx=1.0;
      jp.add(summmulti,jpc);


      jpc.gridy=2;
      jpc.gridx=0;
      jpc.weightx=0.0;
      selregexp=createCheckBox("Text Fragen","CWSELREGEXP",!db.isRegexpExcuded());
      jp.add(selregexp,jpc);
      
      jpc.gridx=1;
      jpc.weightx=1.0;
      jp.add(summregexp,jpc);
    }

    c.gridy=4;
    add(new JLabel(" "),c);
    
    {
      JPanel jp= new JPanel();
      add(jp,c);
      jp.setLayout(new FlowLayout(FlowLayout.LEFT));
      jp.add(new JLabel("Anzahl der Ausgewhlten Fragen: "));
      jp.add(summ);
    }
    
    c.gridy=5;
    selrandq=createCheckBox("Fragen Sortiert","CWSELRANDQ",db.isSortedQuestions());
    add(selrandq,c);
    
    c.gridy=6;
    selranda=createCheckBox("Antworten Sortiert","CWSELRANDQ",db.isSortedAnswers());
    add(selranda,c);

    c.gridy=7;
    c.weighty=1.0;
    c.weightx=1.0;
    add(new JLabel(),c);
    db.sortQuestions();
    db.sortAnswers();
    update_summ();
  }
  
  
  private void update_summ()
  {
    if(db != null)
    {
      boolean si=selsingle.isSelected();
      boolean sm=selmulti.isSelected();
      boolean sr=selregexp.isSelected();
      int min=((Integer)spinnmin.getValue()).intValue()-1;
      int max=((Integer)spinnmax.getValue()).intValue()-1;
    
      if(min>=max)
      {
        spinnmin.setValue(spinnmax.getValue());
        spinnmax.setValue(new Integer(((Integer)spinnmax.getValue()).intValue()+1));
      }
      else if(max <= min)
      {
        spinnmax.setValue(spinnmin.getValue());
        spinnmin.setValue(new Integer(((Integer)spinnmin.getValue()).intValue()-1));
      }
      

      anz.setText(""+db.countAllPossibleQuestions(min,max,true,true,true));
      summsingle.setText(""+db.countAllPossibleQuestions(min,max,true,false,false));
      summmulti.setText(""+db.countAllPossibleQuestions(min,max,false,true,false));
      summregexp.setText(""+db.countAllPossibleQuestions(min,max,false,false,true));
    
      summ.setText(""+db.countAllPossibleQuestions(min,max,si,sm,sr));
    }
  }
  
  private JCheckBox createCheckBox(String txt, String cmd, boolean sel)
  {
    JCheckBox cb = new JCheckBox(txt);
    cb.setActionCommand(cmd);
    cb.addActionListener(this);
    cb.setSelected(sel);
    return cb;
  }

  private JSpinner createSpinner(final String cmd, int now, int min, int max)
  {
    JSpinner sb = new JSpinner();
    sb.setModel(new SpinnerNumberModel(now, min, max, 1));
    sb.addChangeListener(new ChangeListener(){ public void stateChanged(ChangeEvent e) { actionPerformed(new ActionEvent(e.getSource(),0,cmd));}});
    return sb;
  }

  
  public void updateDB()
  {
    db.excludeSingle(!selsingle.isSelected());
    db.excludeMulti(!selmulti.isSelected());
    db.excludeRegexp(!selregexp.isSelected());
    int min=((Integer)spinnmin.getValue()).intValue()-1;
    int max=((Integer)spinnmax.getValue()).intValue()-1;
    db.setRange(min,max);

    if(selrandq.isSelected()) db.sortQuestions();
    else db.randomQuestions();

    if(selranda.isSelected()) db.sortAnswers();
    else db.randomAnswers();
  }
  
  public void actionPerformed(ActionEvent e)
  { update_summ(); }

}
