package gui;

import java.awt.Dimension;
import java.awt.event.*;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.JFileChooser;

import main.Configuration;


import database.DataBase;
import database.Question;

public class GUI implements ActionListener{
  
  private MainWindow win;
  private HelpWindow help;
  private SummaryWindow summ;
  private Question question;
  private JFileChooser filechooser = new JFileChooser();

  
  private ArrayList<ActionListener> action = new ArrayList<ActionListener>();
  private boolean probe=true;
  
  public GUI()
  {
    win = new MainWindow();
    help = new HelpWindow();
    summ= new SummaryWindow();

    win.addActionListener(this);
    help.addActionListener(this);
    summ.addActionListener(this);

    win.endTest();
    hideHelp();
  }
  
  public void setQuestion(Question q){ question=q; }
  public Question getQuestion(){ return question; }
  
  public void startRealTest()
  {
    probe=false;
    win.startTest();
  }
  
  public void startProbeTest()
  {
    win.probeTest();
    probe=true;
    win.startTest();
  }

  public void showQuestion()
  { if(question != null) win.setQuestion(question,probe); }
  
  public void testCorrect()
  {if(question != null && probe) win.testCorrect();}
  
  public void showSummary(){ summ.setVisible(true); }
  public void setSummaryDataBase(DataBase db){ summ.setDataBase(db); }
  public void hideSummary()
  {
    summ.setDataBase(null);
    summ.setVisible(false);
  }

  public void showHelp(Question q)
  {
    help.setQuestion(q);
    help.setVisible(true);
  }
  
  public void hideHelp()
  {
    help.setQuestion(null);
    help.setVisible(false);
  }
  
  public void showConf(Configuration conf)
  {
    new ConfigWindow(win,conf);
    //System.out.println(conf.create());
  }
  
  public void endTest()
  {
    win.endTest();
    hideHelp();
  }
  
  public void disableNext(boolean b){win.disableNext(b);}
  public void disablePrevious(boolean b){win.disablePrevious(b);}
  
  public File openFile()
  {
    if (filechooser.showOpenDialog(win) == JFileChooser.APPROVE_OPTION) return filechooser.getSelectedFile();
    return null;
  }
  
  public File saveAsFile()
  {
    if (filechooser.showSaveDialog(win) == JFileChooser.APPROVE_OPTION) return filechooser.getSelectedFile();
    return null;
  }
  
  public void actionPerformed(ActionEvent e)
  {
    String txt=e.getActionCommand();
    Object o= e.getSource();
    if(txt.equals("THELP"))
    { if(question != null)  showHelp(question); }
    else if(txt.equals("SWHELP"))
    { showHelp((Question)o); }
    else if(txt.equals("SWCLOSE"))
    { hideSummary(); }
    else if(txt.equals("HWCLOSE"))
    { hideHelp(); }
    else
    { doAction(txt,o); }
  }
  
  public void DBConf(DataBase db)
  {
    new DBConfWindow(win,db);
  }
  
  public DataBase DBLoad()
  {
    try {
      return DataBase.loadXDB(new File("FragenTester.xdb"));
    }
    catch (IOException e)
    {
      alert("ERROR LOAD FILE","<html>FragenTester.xdb<br>&nbsp;<br><pre>"+e.getLocalizedMessage()+"</pre></html>","dialog-error");
      return null;
    }
  }
  
  public void alert(String title, String text, String icon)
  { new Alert(win,title,text,LoadIcon.scale(LoadIcon.load(icon),new Dimension(100,100))); }

  public void addActionListener(ActionListener l){action.add(l);}
  public void removeActionListener(ActionListener l){action.remove(l);}
  private void doAction(String name,Object o)
  { for(ActionListener l:action) l.actionPerformed(new ActionEvent(o, 0, name)); }
}
