package gui;

import java.awt.Component;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputMethodEvent;
import java.awt.event.InputMethodListener;

import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextField;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import database.Answer;
import database.Question.Type;

public class ShowAnswers extends JPanel implements ActionListener,InputMethodListener,CaretListener{
  private static final long serialVersionUID = 4415408140549640400L;
  private Answer[] answers;
  private ButtonGroup group = new ButtonGroup();

  ShowAnswers()
  {
    super();
    delete();
  }
  
  void delete()
  {
    setLayout(new GridLayout(1,1));
    add(new Label(""));
  }
  
  void show(Answer[] aa)
  {
    this.answers=aa;
    for(Component c : this.getComponents()) remove(c);
    if(answers != null)
    {
      setLayout(new GridLayout(answers.length,1));
      for(int i=0; i<answers.length; i++)
      {
        Answer a=answers[i];
        if(a.getType()==Type.single)
        {
          add(createRadioButton(a.getText(),""+i,a.getUserSelection()));
        }
        else if(a.getType()==Type.multi)
        {
          add(createCheckBox(a.getText(),""+i,a.getUserSelection()));
        }
        else
        {
          add(createTextField(a.getUserText(),"txt"));
        }
      }
    }
    else
    { delete(); }
  }
  
  private JRadioButton createRadioButton(String txt, String cmd, boolean sel)
  {
    JRadioButton rb = new JRadioButton(Tools.preformatQuestion(txt));
    rb.setActionCommand(cmd);
    rb.addActionListener(this);
    group.add(rb);
    rb.setSelected(sel);
    return rb;
  }
  
  private JCheckBox createCheckBox(String txt, String cmd, boolean sel)
  {
    JCheckBox cb = new JCheckBox(Tools.preformatQuestion(txt));
    cb.setActionCommand(cmd);
    cb.addActionListener(this);
    cb.setSelected(sel);
    return cb;
  }
  
  private JTextField createTextField(String txt, String cmd)
  {
    JTextField text = new JTextField(10);
    if(txt != null) text.setText(txt);
    text.addInputMethodListener(this);
    text.addCaretListener(this);
    text.setActionCommand(cmd);
    text.addActionListener(this);
    return text;
  }
  
  public void actionPerformed(ActionEvent e)
  {
    String cmd=e.getActionCommand();
    if(answers != null)
    {
      Type type= answers[0].getType();
      if(type == Type.single)
      {
        for(int i=0; i<answers.length; i++)
        {
          answers[i].setUserSelection(false);
          if(cmd.equals(""+i))
          { answers[i].setUserSelection(true); }
        }
      }
      else if(type == Type.multi)
      {
        int i=Integer.parseInt(cmd);
        answers[i].setUserSelection(!answers[i].getUserSelection());
      }
      else
      { answers[0].setUserText(((JTextField)e.getSource()).getText()); }
    }
  }
  
  public void caretPositionChanged(InputMethodEvent ime) {}
  public void inputMethodTextChanged(InputMethodEvent ime)
  {
    //System.out.println(((JTextField)ime.getSource()).getText());
    actionPerformed(new ActionEvent(ime.getSource(),0,"txt"));
  }

  public void caretUpdate(CaretEvent e) {
    //System.out.println(((JTextField)e.getSource()).getText());
    actionPerformed(new ActionEvent(e.getSource(),0,"txt"));
  }
}
