package gui;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import javax.swing.*;

import database.*;
import database.Question.Type;

public class SummaryList extends JPanel{
  private static final long serialVersionUID = -2506391788865523138L;
  private GridBagConstraints gbc = new GridBagConstraints();
  private ArrayList<ActionListener> action = new ArrayList<ActionListener>();

  private class myAL implements ActionListener
  {
    String cmd="";
    Question quest;
    myAL(String s, Question q){cmd =s; quest=q;}
    public void actionPerformed(ActionEvent e){ doAction(cmd,quest); }
  }
  
  SummaryList()
  {
    setLayout( new GridBagLayout() );
    cleanList();
    gbc.weighty=1.0;
    gbc.gridy++;
    add(new JLabel(),gbc);
  }
  
  private void cleanList()
  {
    gbc.fill = GridBagConstraints.HORIZONTAL;
    gbc.anchor = GridBagConstraints.FIRST_LINE_START;
    gbc.insets = new Insets(5,5,5,5);
    gbc.gridwidth=0;
    gbc.weightx=1.0;
    gbc.weighty=0.0;
    gbc.gridx=0;
    gbc.gridy=0;
    for(Component c : this.getComponents()) remove(c);
    addDesc();
  }
  
  public void setDataBase(DataBase db)
  {
    cleanList();
    
    int pos=1;
    if(db != null)
    {
      for(Question q: db.getAllPossibleQuestions())
      {
        addQuestion(q,pos);
        pos++;
      }
    }

    gbc.weightx=1.0;
    gbc.weighty=1.0;
    gbc.gridx=0;
    gbc.gridy=pos*2;
    gbc.gridwidth=5;
    add(new JLabel(),gbc);
  }
  
  private void addQuestion(Question q, int pos)
  {
    pos=pos*2;
    JLabel sid = new JLabel(q.getId());

    JLabel icon = new JLabel(LoadIcon.load("ask"));
    if(q.isAnswered())
    {
      if(q.testUserAnswers()) icon = new JLabel(LoadIcon.load("apply"));
      else icon = new JLabel(LoadIcon.load("cancel"));
    }

    JLabel quest = new JLabel(Tools.preformatQuestion(q.getQuestionText()));

    JLabel typ = new JLabel(LoadIcon.load("string"));
    if(q.getType() == Type.single) typ = new JLabel(LoadIcon.load("single"));
    else if(q.getType() == Type.multi) typ = new JLabel(LoadIcon.load("multi"));

    JButton help = new JButton(LoadIcon.load("help"));
    help.addActionListener( new myAL("SWHELP",q) );

    gbc.anchor = GridBagConstraints.FIRST_LINE_START;
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.gridy=pos;
    gbc.gridx=0;
    gbc.gridwidth=1;

    gbc.gridx=0;
    add(sid, gbc);

    gbc.anchor = GridBagConstraints.CENTER;
    gbc.gridx=1;
    add(icon, gbc);
  
    gbc.anchor = GridBagConstraints.FIRST_LINE_START;
    gbc.weightx=1.0;
    gbc.gridx=2;
    add(quest, gbc);
  
    gbc.anchor = GridBagConstraints.CENTER;
    gbc.weightx=0.0;
    gbc.gridx=3;
    add(typ, gbc);

    gbc.anchor = GridBagConstraints.CENTER;
    gbc.gridx=4;
    add(help, gbc);
   
    gbc.weightx=1.0;
    gbc.gridx=0;
    gbc.gridy++;
    gbc.gridwidth=5;
    add(new JSeparator(SwingConstants.HORIZONTAL),gbc);
  }
  
  private void addDesc()
  {
    JLabel sid = new JLabel("ID");
    JLabel icon = new JLabel("Antw.");
    JLabel quest = new JLabel("Frage");
    JLabel typ = new JLabel("Typ");
    JLabel help = new JLabel("Hilfe");

    gbc.anchor = GridBagConstraints.FIRST_LINE_START;
    gbc.weightx=0.0;
    gbc.weighty=0.0;
    gbc.gridx=0;
    gbc.gridy=0;
    gbc.gridwidth=1;

    gbc.gridx=0;
    add(sid, gbc);

    gbc.anchor = GridBagConstraints.CENTER;
    gbc.gridx=1;
    add(icon, gbc);
    
    gbc.anchor = GridBagConstraints.FIRST_LINE_START;
    gbc.weightx=1.0;
    gbc.gridx=2;
    add(quest, gbc);
    
    gbc.anchor = GridBagConstraints.CENTER;
    gbc.weightx=0.0;
    gbc.gridx=3;
    add(typ, gbc);

    gbc.anchor = GridBagConstraints.CENTER;
    gbc.gridx=4;
    add(help, gbc);
    
    gbc.weightx=1.0;
    gbc.gridx=0;
    gbc.gridy=1;
    gbc.gridwidth=5;
    add(new JSeparator(SwingConstants.HORIZONTAL),gbc);
  }
  
  public void addActionListener(ActionListener l){action.add(l);}
  public void removeActionListener(ActionListener l){action.remove(l);}
  private void doAction(String name, Object o)
  {
    for(ActionListener l:action) l.actionPerformed(new ActionEvent(o, 0, name));
  }
}
