package main;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;

import database.DataBase;

public class Configuration {

  private ArrayList<ActionListener> action = new ArrayList<ActionListener>();
  private HashMap<String,HashMap<String,String>> conf = new HashMap<String,HashMap<String,String>>();
  
  //#####################################################
  public Configuration(){ reset(); }
  public Configuration(String f)
  { load(f); }
  //#####################################################
  
  //#####################################################
  public String get(String group,String key)
  {
    if(conf.get(group) == null ) return null;
    return conf.get(group).get(key);
  }
  
  public String[] getGroups()
  { return conf.keySet().toArray(new String[conf.keySet().size()]); }
  
  public String[] getKeys(String group)
  { return conf.get(group).keySet().toArray(new String[conf.get(group).keySet().size()]); }
  
  public int getAsInt(String g,String k)
  {
    if(get(g,k) != null) return Integer.parseInt(get(g,k));
    return 0;
  }
  public boolean getAsBoolean(String g,String k)
  {
    if(get(g,k)!= null) return Boolean.parseBoolean(get(g,k));
    return false;
  }
  public File getAsFile(String g,String k)
  {
   if(get(g,k) != null) return new File(get(g,k));
   return null;
  }
  public String getAsString(String g,String k) { return get(g,k); }
  public double getAsDouble(String g,String k)
  {
    if(get(g,k)!= null) return Double.parseDouble(get(g,k));
    return 0;
  }
  
  public void set(String group,String key,String value)
  {
    if(conf.get(group)==null) conf.put(group, new HashMap<String,String>());
    conf.get(group).put(key,value);
    doAction("CONFCHANGED",this);
  }
  //#####################################################

  
  //#####################################################
  private void reset()
  {
    conf.clear();
    set("conf","file","FragenTester.conf");
    set("db","min","1");
    set("db","max","80");
    set("db","type","file"); // file,sql,net
    set("db","source","FragenTester.db"); // quelle...
    set("db","sort_questions","false");
    set("db","sort_answers","false");
    set("db","select_single","true");
    set("db","select_multi","true");
    set("db","select_regexp","true");
  }

  public void load(){ load(get("conf","file")); }
  public void load(String f)
  {
    reset();
    set("conf","file",f);
    parse(slurpFile(getAsFile("conf","file")));
    doAction("CONFCHANGED",this);
  }
  
  public void save(String f){set("conf","file",f);  save(); }
  public void save(){ if(get("conf","file") != null) writeFile(getAsFile("conf","file"),create()); }
  //#####################################################
  
  //#####################################################
  private void parse(String s)
  {
    //System.out.println("DAT:  "+s);
    String[] list= s.split("[\\r\\n]+");
    String group="global";
    for(String l: list)
    {
      //System.out.println("LN:  "+l);
      if(!l.startsWith("#"))
      {
        if(l.matches(".+=.+"))
        {
          while(l.matches("\\A\\s+.+")) l=l.substring(1); 
          //System.out.println("CNF: "+l);
          l=l.split("#",2)[0];
          String key=l.split("=",2)[0];
          String value=l.split("=",2)[1];
          //System.out.println("CNF: "+group+" -> "+key+" -> "+value);
          set(group,key,value);
        }
        else if(l.matches("^\\s*\\[\\s*\\w+\\s*\\]\\s*$"))
        {
          //System.out.println("GRP: "+l);
          group=l.split("\\[\\s*",2)[1].split("\\s*\\]",2)[0];
        }
      }
    }
  }
  
  public String create()
  {
    String out="";
    for(String group: conf.keySet())
    {
      out+="["+group+"]\n";
      for(String key: conf.get(group).keySet())
      { out+="  "+key+"="+get(group,key)+"\n"; }
    }
    return out;
  }
  //#####################################################
  
  //#####################################################
  private String slurpFile(File file)
  {
    String data="";
    InputStreamReader in=null;
      
    if(file != null) try
    {
      URL rsurl=DataBase.class.getResource("/"+file);
      if(rsurl != null)
        in = new InputStreamReader(rsurl.openStream(),"UTF8");
      else
        in = new InputStreamReader(new FileInputStream(file),"UTF8");
    }
    catch(IOException e)
    { doAction("CONFLOADERROR",e); }
      
    if(in==null)
    { doAction("CONFLOADERROR",new IOException("Datei konnte nicht geffnet werden")); }
    else
    {
      try
      {
        char[] dt = new char[(int)file.length()];
        //System.out.println("lese Datei ... ");
        int ok=in.read(dt,0,(int)file.length());
        //System.out.println("OK="+ok);
        data=String.copyValueOf(dt,0,ok);
        in.close();
        //System.out.println("\nabgeschlossen.");
      }
      catch(IOException e)
      {
        System.out.println(e);
      }
    }
    return data;
  }

  private void writeFile(File f, String s)
  {
    if(f!= null && s!=null)
    {
      try{
        FileWriter fw=new FileWriter(f);
        fw.write(s,0,s.length());
      }catch(IOException e)
      {
        doAction("SAVECONFFAIL",f);
      }
    }
  }
  //#####################################################
  
  //#####################################################
  public void addActionListener(ActionListener l){action.add(l);}
  public void removeActionListener(ActionListener l){action.remove(l);}
  private void doAction(String name,Object o)
  { for(ActionListener l:action) l.actionPerformed(new ActionEvent(o, 0, name)); }
}
